(* runtime configuration *)
(* solyga@gmx.de, 2022-10-01 *)
IMPLEMENTATION MODULE Config;

IMPORT Str, IO, FIO, Lib2, Lib3, Args, SO, Utils;

CONST
  prg = 'LgdRdr';
  ver = 1;
  ext = '2024-09-16';


PROCEDURE WrInfo( VAR b: SO.TBuffer );
BEGIN
  SO.WrStr( b, prg );
  SO.WrStr( b, ' v' );
  SO.WrCard( b, ver, 0 );
  SO.WrStr( b, ' (' );
  SO.WrStr( b, ext );
  SO.WrStr( b, '): LGD file reader.' );
END WrInfo;


PROCEDURE ParseArgs();

  PROCEDURE Help();
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.Clear( b ); SO.WrStr( b, "(* " ); WrInfo( b ); SO.WrStr( b, " *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "syntax = '" ); SO.WrStr( b, prg ); SO.WrStr( b, "' { option } [ infile [ outfile ] ] ." );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "option = '-h'      (* show this info and exit *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "       | '-d'      (* switch on debug mode *)" );
    IO.WrStr( s ); IO.WrLn();
    SO.Clear( b ); SO.WrStr( b, "       | '-v'      (* switch on verbose mode *) ." );
    IO.WrStr( s ); IO.WrLn();
    Lib3.SetReturnCode( 0 );
    HALT;
  END Help;

  PROCEDURE Error( t-: ARRAY OF CHAR );
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.WrStr( b, 'Error: ' );
    SO.WrStr( b, t );
    IF Args.len > 0 THEN
      SO.WrStr( b, ": '" );
      SO.WrStr( b, Args.lit );
      SO.WrStr( b, "'" );
    END;
    SO.WrLn( b );
    SO.WrStr( b, "Try '" );
    SO.WrStr( b, prg );
    SO.WrStr( b, " -h' for help." );
    Lib3.FatalError( s );
  END Error;

  PROCEDURE GetAddress( VAR x: Types.TLngCard );
  BEGIN
    IF ( Args.sym # Args.symArg ) OR ~ Utils.MkIP( x, Args.lit ) THEN Error( 'Invalid IP address' ) END;
  END GetAddress;

  PROCEDURE GetPort( VAR p: Types.TCard );
  VAR
    x: Types.TLngCard;
    ok: BOOLEAN;
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Invalid port number' ) END;
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok THEN Error( 'Invalid port number' ) END;
    IF x > MAX( Types.TCard ) THEN Error( 'Port number out of range' ) END;
    p := Types.TCard( x );
  END GetPort;

  PROCEDURE GetCard( VAR x: LONGCARD );
  VAR
    ok: BOOLEAN;
  BEGIN
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok THEN Error( 'Invalid number' ) END;
  END GetCard;

  PROCEDURE Common( s-, t-: ARRAY OF CHAR ): CARDINAL;
  VAR
    l, i: CARDINAL;
  BEGIN
    l := Str.Length( s );
    i := Str.Length( t );
    IF l > i THEN l := i END;
    i := 0;
    WHILE i < l DO
      IF CAP( s[i] ) # CAP( t[i] ) THEN RETURN i END;
      INC( i );
    END;
    RETURN l;
  END Common;

  PROCEDURE GetNum( VAR x: ARRAY OF CHAR );
  BEGIN
    Str.Copy( x, Args.lit );
  END GetNum;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF    Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'd' ) = 0 THEN
        data.dbg := TRUE;
      ELSIF Str.Compare( Args.lit, 'v' ) = 0 THEN
        data.vrb := TRUE;
      ELSE
        Error( 'Invalid option' );
      END;
      Args.Read();
    END;
  END Options;

BEGIN
  Args.Reset();
  Args.Read();
  Options();
  IF Args.sym = Args.symArg THEN
    Str.Copy( data.in, Args.lit );
    Args.Read();
    Options();
    IF Args.sym = Args.symArg THEN
      Str.Copy( data.out, Args.lit );
      Args.Read();
      Options();
    END;
  END;
  IF Args.sym # Args.symEOA THEN Error( 'Unexpected argument' ) END;
END ParseArgs;

PROCEDURE InitDir();
VAR
  drv: SHORTCARD;
BEGIN
  drv := FIO.GetDrive();
  FIO.GetDir( drv, data.dir );
  Str.Prepend( data.dir, ':' );
  Str.Prepend( data.dir, CHR( ORD( 'A' ) + CARDINAL(drv-1) ) );
END InitDir;

PROCEDURE Show();
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;

  PROCEDURE Out();
  BEGIN
    (* must be 2x IO (not FIO) for redirected stream beeing correct *)
    IO.WrStr( s );
    IO.WrLn();
  END Out;

  PROCEDURE ShowHdr();
  BEGIN
    SO.Clear( b ); WrInfo( b ); Out();
    SO.Clear( b ); SO.WrStr( b, 'config' ); Out();
  END ShowHdr;

  PROCEDURE Begin( n-: ARRAY OF CHAR );
  CONST w = 3;
  BEGIN
    SO.Clear( b ); SO.WrStr( b, '  ' ); SO.WrStrAdj( b, n, -w ); SO.WrChar( b, ' ' );
  END Begin;

  PROCEDURE ShowStr( n-: ARRAY OF CHAR; v-: ARRAY OF CHAR );
  BEGIN
    Begin( n ); SO.WrChar( b, "'" ); SO.WrStr( b, v ); SO.WrChar( b, "'" ); Out();
  END ShowStr;

  PROCEDURE ShowCard( n-: ARRAY OF CHAR; v: CARDINAL );
  CONST w = 1;
  BEGIN
    Begin( n ); SO.WrLngCard( b, v, w ); Out();
  END ShowCard;

  PROCEDURE ShowIP( n-: ARRAY OF CHAR; x: Types.TLngCard );
  BEGIN
    Begin( n ); Utils.WrIP( b, x ); Out();
  END ShowIP;

BEGIN
  IF ~ data.vrb THEN RETURN END;
  SO.Init( b, s );
  ShowHdr();
  ShowStr(  'prg', data.prg );
  ShowCard( 'ver', data.ver );
  ShowStr(  'ext', data.ext );
  ShowCard( 'dbg', ORD( data.dbg ) );
  ShowCard( 'vrb', ORD( data.vrb ) );
  ShowStr(  'dir', data.dir );
  ShowStr(  'in ', data.in );
  ShowStr(  'out', data.out );
END Show;

PROCEDURE InitModule();
BEGIN
  Lib2.Fill( data, 0 );
  data.prg := prg;
  data.ver := ver;
  data.ext := ext;
  data.in := 'tst.lgd';
  data.out := '';
  InitDir();
  ParseArgs();
  Show();
END InitModule;

BEGIN
  InitModule();
END Config.
